#!/usr/bin/perl -w
# $Id: PFIF.pm,v 1.6 2005/09/09 23:54:49 ianb Exp $
# Ian Beckwith <ianb@nessie.mcc.ac.uk>
#


package XML::PFIF;
use strict;
use POSIX;

use vars qw($VERSION);
$VERSION = '0.2'; # Version of module

=head1 NAME

XML::PFIF - create PFIF (People Finder Interchange Format) XML and RSS files

=head1 DESCRIPTION

XML::PFIF is a Perl module for creating PFIF (People Finder
Interchange Format) XML and RSS files, for the exchange of
data about missing persons.

XML::PFIF currently implements version 1.1 of the PFIF specification.

There are two classes, implementing types of records. Person records
(L<XML::PFIF::Person|XML::PFIF::Person>) are for static
information. Note records (L<XML::PFIF::Note|XML::PFIF::Note>) are for
changing information. Each note record belongs to a particular person,
and a person record may have any number of associated note records.

The XML::PFIF class is a parent class to both these classes,
and provides various helper functions used by both classes.    
It is unlikely to be much use by itself.

See L<XML::PFIF::Person> and L<XML::PFIF::Note> for details on use of
this module, along with the files B<testpfif> and B<testrss>.

=head1 METHODS

=cut    

#don't really need new, don't document?    

sub new
{
    my $proto=shift;
    my $class=ref($proto) || $proto;
    my $self={};
    bless($self,$class);

    # FIXME:  not yet used
    # name of scraper or undef if not automated
    # $self->{automated}=shift; 
    
    return $self;
}


=head3 start_xml()

Returns a string containing the XML header that should start all PFIF
XML and RSS files.

=cut

sub start_xml
{
    return "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n";
}

=head3 start_rss()

Returns a string containing the xml snippet to start a PFIF RSS feed.

=cut

sub start_rss
{
    my $str=<<'EOT';
<rss version="2.0"
     xmlns:pfif="http://zesty.ca/pfif/1.1"
     xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
     xsi:schemaLocation="http://zesty.ca/pfif/1.1
                         http://zesty.ca/pfif/1.1/pfif-1.1.xsd">
EOT
    return $str;
}

=head3 end_rss()

Returns a string containing the xml snippet to end a PFIF RSS feed.

=cut

sub end_rss
{
    return "</rss>\n";
}


=head3 start_channel()

Returns a string containing the xml snippet to start a PFIF RSS
channel.

=cut

sub start_channel
{
    my ($self,$hash)=@_;
    my $str= "  <channel>\n";
    my $indent="    ";
    for my $field (@{$self->{channelfields}})
    {
	if(defined($hash) && exists($hash->{$field}))
	{
	    $self->{$field}=$hash->{$field};
	}
	else
	{
	    $self->{$field}='';
	}

	$str .=  $indent . $self->tag($field,$self->{$field});
    }
    return $str;
}


=head3 end_channel()

Returns a string containing the xml snippet to end a channel.

=cut

sub end_channel
{
    return "  </channel>\n";
}

=head3 start_pfif()

Returns a string containing the xml snippet to start a PFIF
container. Not needed with RSS.

=cut

sub start_pfif
{
    my($self)=(@_);
    my $str=<<'EOT';
<pfif:pfif xmlns:pfif="http://zesty.ca/pfif/1.1"
           xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
           xsi:schemaLocation="http://zesty.ca/pfif/1.1
                               http://zesty.ca/pfif/1.1/pfif-1.1.xsd">
EOT
    return $str;
}

=head3 end_pfif()

Returns a string containing the xml snippet to end a PFIF
container. Not needed with RSS.

=cut

sub end_pfif
{
    return "</pfif:pfif>\n";
}

=head3 tag($tag, $value)

Returns a string containing $tag printed as xml, either with $value
filled in or, if empty, as a self-closing <tag/>.

=cut

sub tag
{
    my($self,$tag,$val)=@_;
    my $str='';
    if($val ne '')
    {
	$str="<$tag>$val</$tag>\n";
    }
    else
    {
	$str="<$tag/>\n";
    }
    return $str;
}


=head3 date_to_rfc822($date)

Converts $date which should be in the format "yyyy-mm-ddThh:mm:ssZ"
to a rfc822-compliant date for use in the rss <channel> pubDate attribute.

=cut

sub date_to_rfc822
{
    my($self,$indate)=@_;
    if($indate=~/^(\d\d\d\d)-(\d\d)-(\d\d)T(\d\d):(\d\d):(\d\d)Z/)
    {
	my($year,$month,$day,$hour,$min,$sec)=($1,$2,$3,$4,$5,$6);
	return POSIX::strftime("%a, %d %b %Y %H:%M:%S GMT",
			       $sec, $min, $hour, $day, ($month-1), ($year-1900));
    }
    return '';
}

=head3 now()

Returns the current date and time in "yyyy-mm-ddThh:mm:ssZ"
format.    

=cut

sub now
{
    return POSIX::strftime("%Y-%m-%dT%H:%M:%SZ",gmtime(time()));
}


=head3 date_to_human($date)

Converts $date which should be in the format "yyyy-mm-ddThh:mm:ssZ"
to a human-readable format.

=cut

sub date_to_human
{
    my($self,$indate)=@_;
    if($indate=~/^(\d\d\d\d)-(\d\d)-(\d\d)T(\d\d):(\d\d):(\d\d)Z/)
    {
	my($year,$month,$day,$hour,$min,$sec)=($1,$2,$3,$4,$5,$6);
	return POSIX::strftime("%H:%M:%S GMT on %Y-%m-%d",
			       $sec, $min, $hour, $day, ($month-1), ($year-1900));
    }
    return '';
}


=head3 validate_date($date)

Returns 1 if the date is in a valid "yyyy-mm-ddThh:mm:ssZ"
format, else warn()s of the error and returns 0;

=cut

sub validate_date
{
    my($self,$date)=@_;
    if($date=~/^(\d\d\d\d)-(\d\d)-(\d\d)T(\d\d):(\d\d):(\d\d)Z/)
    {
	my($years,$months,$days,$hours,$mins,$secs)=($1,$2,$3,$4,$5,$6);
	if(($months < 1) || ($months > 12))
	{
	    warn("validate_date: bad month: $months\n");
	    return 0;
	}
	if(($days < 1) || ($days > 31))
	{
	    warn("validate_date: bad day: $days\n");
	    return 0;
	}
	if(($hours < 0) || ($hours > 24))
	{
	    warn("validate_date: bad hour: $hours\n");
	    return 0;
	}
	if(($days < 1) || ($days > 31))
	{
	    warn("validate_date: bad day: $days\n");
	    return 0;
	}
	if(($mins < 0) || ($mins > 60))
	{
	    warn("validate_date: bad minute: $mins\n");
	    return 0;
	}
	if(($secs < 0) || ($secs > 60))
	{
	    warn("validate_date: bad second: $secs\n");
	    return 0;
	}
    }
    else
    {
	warn("validate_date: bad date format: expecting yyyy-mm-ddThh:mm:ssZ\n");
	return 0;
    }
	     
    return 1;
}

=head3 fixup_string($string)

Converts $string to the canonical format for searchable fields.
Accents are converted to their non-accent equivalents, and letters are
upcased.

=cut

sub fixup_string
{
    my ($self,$str)=@_;

    my $achars="";
    my $aechars="";
    my $bchars="";
    my $cchars="";
    my $echars="";
    my $ichars="";
    my $nchars="";
    my $ochars="";
    my $uchars="";
    my $ychars="";

    $str=~s/[$achars]/A/g;
    $str=~s/[$aechars]/AE/g;
    $str=~s/[$bchars]/B/g;
    $str=~s/[$cchars]/C/g;
    $str=~s/[$echars]/E/g;
    $str=~s/[$ichars]/I/g;
    $str=~s/[$nchars]/N/g;
    $str=~s/[$ochars]/O/g;
    $str=~s/[$uchars]/U/g;
    $str=~s/[$ychars]/Y/g;

    $str=~tr/[a-z]/[A-Z]/;

    # fix escaped ampersands. Is this sufficient or will other things crop up?
    $str=~s/&AMP;/&amp;/g;

    return $str;

}

1;

__END__


=head1 BUGS

Please report any found to ianb@nessie.mcc.ac.uk	

=head1 EXAMPLES

See B<testpfif> and B<testrss> with the XML::PFIF distribution,
which (mostly) reproduce the example XML and RSS files at
L<http://zesty.ca/pfif/>    

=head1 SEE ALSO    

L<XML::PFIF::Person>, L<XML::PFIF::Note>,

PFIF Specification: L<http://zesty.ca/pfif/>

Katrina PeopleFinder Project wiki Tech page: L<http://katrina.internet2.edu/wiki/>

=head1 AUTHOR

Ian Beckwith <ianb@nessie.mcc.ac.uk>

=head1 AVAILABILITY

The latest version can be found at:

B<http://nessie.mcc.ac.uk/~ianb/projects/pfif/>

=head1 COPYRIGHT

Copyright 2005 Ian Beckwith <ianb@nessie.mcc.ac.uk>

This code is in the Public Domain.

If you need a license other than Public Domain,
then consider it under the BSD License:

 Copyright (c) Ian Beckwith
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
 3. Neither the name of the University nor the names of its contributors
    may be used to endorse or promote products derived from this software
    without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE
 FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 SUCH DAMAGE.

=cut


